@@ -31,6 +31,8 @@ import org.apache.lucene.util.automaton.CompiledAutomaton;
  * allows for a {@link QueryTimeout} implementation object to be checked periodically
  * to see if the thread should exit or not.  If {@link QueryTimeout#shouldExit()}
  * returns true, an {@link ExitingReaderException} is thrown.
+ *
+ * @see org.apache.lucene.search.TimeLimitingCollector
  */
 public class ExitableDirectoryReader extends FilterDirectoryReader {
   
@@ -43,7 +45,7 @@ public class ExitableDirectoryReader extends FilterDirectoryReader {
   public static class ExitingReaderException extends RuntimeException {
 
     /** Constructor **/
-    ExitingReaderException(String msg) {
+    public ExitingReaderException(String msg) {
       super(msg);
     }
   }
@@ -494,35 +496,39 @@ public class ExitableDirectoryReader extends FilterDirectoryReader {
    * exitable enumeration of terms.
    */
   public static class ExitableTermsEnum extends FilterTermsEnum {
-
+    // Create bit mask in the form of 0000 1111 for efficient checking
+    private static final int NUM_CALLS_PER_TIMEOUT_CHECK = (1 << 4) - 1; // 15
+    private int calls;
     private QueryTimeout queryTimeout;
     
     /** Constructor **/
     public ExitableTermsEnum(TermsEnum termsEnum, QueryTimeout queryTimeout) {
       super(termsEnum);
       this.queryTimeout = queryTimeout;
-      checkAndThrow();
+      checkTimeoutWithSampling();
     }
 
     /**
      * Throws {@link ExitingReaderException} if {@link QueryTimeout#shouldExit()} returns true,
      * or if {@link Thread#interrupted()} returns true.
      */
-    private void checkAndThrow() {
-      if (queryTimeout.shouldExit()) {
-        throw new ExitingReaderException("The request took too long to iterate over terms. Timeout: " 
-            + queryTimeout.toString()
-            + ", TermsEnum=" + in
-        );
-      } else if (Thread.interrupted()) {
-        throw new ExitingReaderException("Interrupted while iterating over terms. TermsEnum=" + in);
+    private void checkTimeoutWithSampling() {
+      if ((calls++ & NUM_CALLS_PER_TIMEOUT_CHECK) == 0) {
+        if (queryTimeout.shouldExit()) {
+          throw new ExitingReaderException("The request took too long to iterate over terms. Timeout: "
+              + queryTimeout.toString()
+              + ", TermsEnum=" + in
+          );
+        } else if (Thread.interrupted()) {
+          throw new ExitingReaderException("Interrupted while iterating over terms. TermsEnum=" + in);
+        }
       }
     }
 
     @Override
     public BytesRef next() throws IOException {
       // Before every iteration, check if the iteration should exit
-      checkAndThrow();
+      checkTimeoutWithSampling();
       return in.next();
     }
   }
