@@ -110,8 +110,8 @@ public final class IndexWriterConfig extends LiveIndexWriterConfig {
   /** Default value for whether calls to {@link IndexWriter#close()} include a commit. */
   public final static boolean DEFAULT_COMMIT_ON_CLOSE = true;
 
-  /** Default value for time to wait for merges on commit (when using a {@link MergePolicy} that implements {@link MergePolicy#findFullFlushMerges}). */
-  public static final long DEFAULT_MAX_COMMIT_MERGE_WAIT_MILLIS = 0;
+  /** Default value for time to wait for merges on commit or getReader (when using a {@link MergePolicy} that implements {@link MergePolicy#findFullFlushMerges}). */
+  public static final long DEFAULT_MAX_FULL_FLUSH_MERGE_WAIT_MILLIS = 0;
   
   // indicates whether this config instance is already attached to a writer.
   // not final so that it can be cloned properly.
@@ -463,17 +463,18 @@ public final class IndexWriterConfig extends LiveIndexWriterConfig {
   }
 
   /**
-   * Expert: sets the amount of time to wait for merges (during {@link IndexWriter#commit}) returned by
+   * Expert: sets the amount of time to wait for merges (during {@link IndexWriter#commit}
+   * or {@link IndexWriter#getReader(boolean, boolean)}) returned by
    * MergePolicy.findFullFlushMerges(...).
    * If this time is reached, we proceed with the commit based on segments merged up to that point.
-   * The merges are not cancelled, and will still run to completion independent of the commit,
-   * like natural segment merges. The default is <code>{@value IndexWriterConfig#DEFAULT_MAX_COMMIT_MERGE_WAIT_MILLIS}</code>.
+   * The merges are not aborted, and will still run to completion independent of the commit or getReader call,
+   * like natural segment merges. The default is <code>{@value IndexWriterConfig#DEFAULT_MAX_FULL_FLUSH_MERGE_WAIT_MILLIS}</code>.
    *
    * Note: This settings has no effect unless {@link MergePolicy#findFullFlushMerges(MergeTrigger, SegmentInfos, MergePolicy.MergeContext)}
    * has an implementation that actually returns merges which by default doesn't return any merges.
    */
-  public IndexWriterConfig setMaxCommitMergeWaitMillis(long maxCommitMergeWaitMillis) {
-    this.maxCommitMergeWaitMillis = maxCommitMergeWaitMillis;
+  public IndexWriterConfig setMaxFullFlushMergeWaitMillis(long maxFullFlushMergeWaitMillis) {
+    this.maxFullFlushMergeWaitMillis = maxFullFlushMergeWaitMillis;
     return this;
   }
 
