@@ -245,11 +245,25 @@ public final class Version {
 
   /**
    * Match settings and bugs in Lucene's 8.7.0 release.
+   * @deprecated (8.8.0) Use latest
+   */
+  @Deprecated
+  public static final Version LUCENE_8_7_0 = new Version(8, 7, 0);
+
+  /**
+   * Match settings and bugs in Lucene's 8.8.0 release.
+   * @deprecated (8.8.1) Use latest
+   */
+  @Deprecated
+  public static final Version LUCENE_8_8_0 = new Version(8, 8, 0);
+
+  /**
+   * Match settings and bugs in Lucene's 8.8.1 release.
    * <p>
    * Use this to get the latest &amp; greatest settings, bug
    * fixes, etc, for Lucene.
    */
-  public static final Version LUCENE_8_7_0 = new Version(8, 7, 0);
+  public static final Version LUCENE_8_8_1 = new Version(8, 8, 1);
 
   // To add a new version:
   //  * Only add above this comment
@@ -270,7 +284,7 @@ public final class Version {
    * some defaults may have changed and may break functionality
    * in your application.
    */
-  public static final Version LATEST = LUCENE_8_7_0;
+  public static final Version LATEST = LUCENE_8_8_1;
 
   /**
    * Constant for backwards compatibility.
