@@ -21,10 +21,7 @@ import java.util.List;
 
 import org.apache.lucene.document.ShapeField.QueryRelation;
 import org.apache.lucene.geo.Component2D;
-import org.apache.lucene.geo.Tessellator;
 import org.apache.lucene.geo.XYPolygon;
-import org.apache.lucene.geo.XYRectangle;
-import org.apache.lucene.geo.XYGeometry;
 import org.apache.lucene.util.LuceneTestCase;
 
 /** random cartesian bounding box, line, and polygon query tests for random indexed arrays of cartesian {@link XYPolygon} types */
@@ -40,47 +37,11 @@ public class TestXYMultiPolygonShapeQueries extends BaseXYShapeTestCase {
     int n = random().nextInt(4) + 1;
     XYPolygon[] polygons = new XYPolygon[n];
     for (int i =0; i < n; i++) {
-      int  repetitions =0;
-      while (true) {
-        // if we can't tessellate; then random polygon generator created a malformed shape
-        XYPolygon p = (XYPolygon) getShapeType().nextShape();
-        try {
-          Tessellator.tessellate(p);
-          //polygons are disjoint so CONTAINS works. Note that if we intersect
-          //any shape then contains return false.
-          if (isDisjoint(polygons, p, i)) {
-            polygons[i] = p;
-            break;
-          }
-          repetitions++;
-          if (repetitions > 2) {
-            //try again
-            return nextShape();
-          }
-        } catch (IllegalArgumentException e) {
-          continue;
-        }
-      }
+      polygons[i] =  (XYPolygon) getShapeType().nextShape();
     }
     return polygons;
   }
-
-  private boolean isDisjoint(XYPolygon[] polygons, XYPolygon check, int totalPolygons) {
-    // we use bounding boxes so we do not get polygons with shared points.
-    for (XYPolygon polygon : polygons) {
-      if (polygon != null) {
-        if (getEncoder().quantizeY(polygon.minY) > getEncoder().quantizeY(check.maxY)
-            || getEncoder().quantizeY(polygon.maxY) < getEncoder().quantizeY(check.minY)
-            || getEncoder().quantizeX(polygon.minX) > getEncoder().quantizeX(check.maxX)
-            || getEncoder().quantizeX(polygon.maxX) < getEncoder().quantizeX(check.minX)) {
-          continue;
-        }
-        return false;
-      }
-    }
-    return true;
-  }
-
+  
   @Override
   protected Field[] createIndexableFields(String name, Object o) {
     XYPolygon[] polygons = (XYPolygon[]) o;
@@ -114,12 +75,6 @@ public class TestXYMultiPolygonShapeQueries extends BaseXYShapeTestCase {
       return this;
     }
 
-    @Override
-    public boolean testBBoxQuery(double minY, double maxY, double minX, double maxX, Object shape) {
-      Component2D rectangle2D = XYGeometry.create(new XYRectangle((float) minX, (float) maxX, (float) minY, (float) maxY));
-      return testComponentQuery(rectangle2D, shape);
-    }
-
     @Override
     public boolean testComponentQuery(Component2D query, Object shape) {
       XYPolygon[] polygons = (XYPolygon[]) shape;
