@@ -115,6 +115,12 @@ public class TestDrillSideways extends FacetTestCase {
     };
   }
 
+  private IndexSearcher getNewSearcher(IndexReader reader) {
+    // Do not wrap with an asserting searcher, since DrillSidewaysQuery doesn't
+    // implement all the required components like Weight#scorer.
+    return newSearcher(reader, true, false);
+  }
+
   public void testBasic() throws Exception {
     Directory dir = newDirectory();
     Directory taxoDir = newDirectory();
@@ -154,7 +160,7 @@ public class TestDrillSideways extends FacetTestCase {
     writer.addDocument(config.build(taxoWriter, doc));
 
     // NRT open
-    IndexSearcher searcher = newSearcher(writer.getReader());
+    IndexSearcher searcher = getNewSearcher(writer.getReader());
 
     //System.out.println("searcher=" + searcher);
 
@@ -339,7 +345,7 @@ public class TestDrillSideways extends FacetTestCase {
     writer.addDocument(config.build(taxoWriter, doc));
 
     // NRT open
-    IndexSearcher searcher = newSearcher(writer.getReader());
+    IndexSearcher searcher = getNewSearcher(writer.getReader());
 
     //System.out.println("searcher=" + searcher);
 
@@ -402,7 +408,7 @@ public class TestDrillSideways extends FacetTestCase {
     writer.addDocument(config.build(taxoWriter, doc));
 
     // NRT open
-    IndexSearcher searcher = newSearcher(writer.getReader());
+    IndexSearcher searcher = getNewSearcher(writer.getReader());
 
     //System.out.println("searcher=" + searcher);
 
@@ -629,7 +635,7 @@ public class TestDrillSideways extends FacetTestCase {
     IndexReader r = w.getReader();
 
     final SortedSetDocValuesReaderState sortedSetDVState;
-    IndexSearcher s = newSearcher(r);
+    IndexSearcher s = getNewSearcher(r);
 
     if (doUseDV) {
       sortedSetDVState = new DefaultSortedSetDocValuesReaderState(s.getIndexReader());
@@ -1139,7 +1145,7 @@ public class TestDrillSideways extends FacetTestCase {
     Directory taxoDir = newDirectory();
     RandomIndexWriter writer = new RandomIndexWriter(random(), dir);
     DirectoryTaxonomyWriter taxoWriter = new DirectoryTaxonomyWriter(taxoDir, IndexWriterConfig.OpenMode.CREATE);
-    IndexSearcher searcher = newSearcher(writer.getReader());
+    IndexSearcher searcher = getNewSearcher(writer.getReader());
     TaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
 
     // Count "Author"
@@ -1178,7 +1184,7 @@ public class TestDrillSideways extends FacetTestCase {
     writer.addDocument(config.build(taxoWriter, doc));
 
     // NRT open
-    IndexSearcher searcher = newSearcher(writer.getReader());
+    IndexSearcher searcher = getNewSearcher(writer.getReader());
 
     // NRT open
     TaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
